@@ -1,6 +1,7 @@
-/* $Id: main.c 27686 2008-09-06 12:21:41Z stephan $ */
+/* $Id: main.c 28142 2008-10-11 11:10:44Z kelnos $ */
 /*-
  * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,10 +46,16 @@
 #include <unistd.h>
 #endif
 
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
+#include <xfconf/xfconf.h>
+
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
+#include <libxfcegui4/libxfcegui4.h>
 
 #include <libxfsm/xfsm-util.h>
 
@@ -59,7 +66,7 @@
 #include <xfce4-session/xfsm-global.h>
 #include <xfce4-session/xfsm-manager.h>
 #include <xfce4-session/xfsm-startup.h>
-
+#include "xfsm-error.h"
 
 void
 setup_environment (void)
@@ -125,22 +132,21 @@ usage (int exit_code)
 
 
 static void
-init_display (GdkDisplay *dpy,
-              XfceRc     *rc,
-              gboolean disable_tcp)
+init_display (XfsmManager   *manager,
+              GdkDisplay    *dpy,
+              XfconfChannel *channel,
+              gboolean       disable_tcp)
 {
   const gchar *engine;
 
-  xfce_rc_set_group (rc, "Splash Screen");
-  engine = xfce_rc_read_entry (rc, "Engine", NULL);
+  engine = xfconf_channel_get_string (channel, "/splash/Engine", "mice");
 
   splash_screen = xfsm_splash_screen_new (dpy, engine);  
   xfsm_splash_screen_next (splash_screen, _("Loading desktop settings"));
 
   gdk_flush ();
 
-  xfce_rc_set_group (rc, "General");
-  sm_init (rc, disable_tcp);
+  sm_init (channel, disable_tcp, manager);
 
   /* start xfsettingsd */
   if ( !g_spawn_command_line_async ("xfsettingsd", NULL))
@@ -155,11 +161,13 @@ init_display (GdkDisplay *dpy,
 
 
 static void
-initialize (int argc, char **argv)
+initialize (XfsmManager *manager,
+            int          argc,
+            char       **argv)
 {
   gboolean disable_tcp = FALSE;
   GdkDisplay *dpy;
-  XfceRc *rc;
+  XfconfChannel *channel;
   
   for (++argv; --argc > 0; ++argv)
     {
@@ -189,10 +197,10 @@ initialize (int argc, char **argv)
 
   setup_environment ();
 
-  rc = xfsm_open_config (TRUE);
+  channel = xfsm_open_config ();
 
   dpy = gdk_display_get_default ();
-  init_display (dpy, rc, disable_tcp);
+  init_display (manager, dpy, channel, disable_tcp);
 
   /* verify that the DNS settings are ok */
   xfsm_splash_screen_next (splash_screen, _("Verifying DNS settings"));
@@ -200,31 +208,61 @@ initialize (int argc, char **argv)
 
   xfsm_splash_screen_next (splash_screen, _("Loading session data"));
 
-  xfce_rc_set_group (rc, "Compatibility");
-  compat_gnome = xfce_rc_read_bool_entry (rc, "LaunchGnome", FALSE);
-  compat_kde = xfce_rc_read_bool_entry (rc, "LaunchKDE", FALSE);
-
-  xfce_rc_set_group (rc, "General");
-  xfsm_startup_init (rc);
-  xfsm_manager_init (rc);
-
-  /* cleanup obsolete entries */
-  xfce_rc_set_group (rc, "General");
-  if (xfce_rc_has_entry (rc, "ConfirmLogout"))
-    xfce_rc_delete_entry (rc, "ConfirmLogout", FALSE);
-  if (xfce_rc_has_entry (rc, "AlwaysDisplayChooser"))
-    xfce_rc_delete_entry (rc, "AlwaysDisplayChooser", FALSE);
-  xfce_rc_delete_group (rc, "Splash Theme", FALSE);
-
-  xfce_rc_close (rc);
+  xfsm_startup_init (channel);
+  xfsm_manager_load (manager, channel);
 }
 
 
+void
+xfsm_dbus_init (void)
+{
+  DBusGConnection *dbus_conn;
+  int              ret;
+  GError          *error = NULL;
+
+  xfsm_error_dbus_init ();
+
+  dbus_conn = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+  if (G_UNLIKELY (!dbus_conn))
+    {
+      g_critical ("Unable to contact D-Bus session bus: %s", error ? error->message : "Unknown error");
+      if (error)
+        g_error_free (error);
+      return;
+    }
+
+  ret = dbus_bus_request_name (dbus_g_connection_get_connection (dbus_conn),
+                               "org.xfce.SessionManager",
+                               DBUS_NAME_FLAG_DO_NOT_QUEUE,
+                               NULL);
+  if (DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER != ret)
+    {
+      g_error ("Another session manager is already running");
+      exit (1);
+    }
+}
+
+void
+xfsm_dbus_cleanup (void)
+{
+  DBusGConnection *dbus_conn;
+
+  /* this is all not really necessary, but... */
+
+  dbus_conn = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+  if (G_UNLIKELY (!dbus_conn))
+    return;
+
+  dbus_bus_release_name (dbus_g_connection_get_connection (dbus_conn),
+                         "org.xfce.SessionManager", NULL);
+}
+
 int
 main (int argc, char **argv)
 {
-  /* imported from xfsm-manager.c */
-  extern gint shutdown_type;
+  XfsmManager     *manager;
+  XfsmShutdownType shutdown_type;
+  GError          *error = NULL;
 
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
   
@@ -233,17 +271,33 @@ main (int argc, char **argv)
 
   gtk_init (&argc, &argv);
 
+  if (G_UNLIKELY (!xfconf_init (&error))) {
+    xfce_message_dialog (NULL, _("Xfce Session Manager"),
+                         GTK_STOCK_DIALOG_ERROR,
+                         _("Unable to contact settings server"),
+                         error->message,
+                         GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
+                         NULL);
+    g_error_free (error);
+  }
+
   /* fake a client id for the manager, so the legacy management does not
    * recognize us to be a session client.
    */
-  gdk_set_sm_client_id (xfsm_manager_generate_client_id (NULL));
+  gdk_set_sm_client_id (xfsm_generate_client_id (NULL));
 
-  initialize (argc, argv);
-  
-  xfsm_manager_restart ();
+  xfsm_dbus_init ();
+
+  manager = xfsm_manager_new ();
+  initialize (manager, argc, argv);
+  xfsm_manager_restart (manager);
   
   gtk_main ();
-  
+
+  shutdown_type = xfsm_manager_get_shutdown_type (manager);
+  g_object_unref (manager);
+ 
+  xfsm_dbus_cleanup ();
   ice_cleanup ();
 
   return xfsm_shutdown (shutdown_type);
