@@ -1,4 +1,4 @@
-/* $Id: shutdown.c 27022 2008-06-03 17:45:59Z stephan $ */
+/* $Id: shutdown.c 28142 2008-10-11 11:10:44Z kelnos $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -40,6 +40,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_ASM_UNISTD_H
+#include <asm/unistd.h>  /* for __NR_ioprio_set */
+#endif
+
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <gtk/gtk.h>
@@ -63,7 +67,7 @@ static GtkWidget *shutdown_dialog = NULL;
 
 #ifdef SESSION_SCREENSHOTS
 static void
-screenshot_save (GdkPixmap *pm, GdkRectangle *area)
+screenshot_save (const gchar *session_name, GdkPixmap *pm, GdkRectangle *area)
 {
   gchar *display_name;
   gchar *resource;
@@ -113,7 +117,7 @@ entry_activate_cb (GtkWidget *entry, GtkDialog *dialog)
 static void
 logout_button_clicked (GtkWidget *b, gint *shutdownType)
 {
-    *shutdownType = SHUTDOWN_LOGOUT;
+    *shutdownType = XFSM_SHUTDOWN_LOGOUT;
 
     gtk_dialog_response (GTK_DIALOG (shutdown_dialog), GTK_RESPONSE_OK);
 }
@@ -121,7 +125,7 @@ logout_button_clicked (GtkWidget *b, gint *shutdownType)
 static void
 reboot_button_clicked (GtkWidget *b, gint *shutdownType)
 {
-    *shutdownType = SHUTDOWN_REBOOT;
+    *shutdownType = XFSM_SHUTDOWN_REBOOT;
 
     gtk_dialog_response (GTK_DIALOG (shutdown_dialog), GTK_RESPONSE_OK);
 }
@@ -129,7 +133,7 @@ reboot_button_clicked (GtkWidget *b, gint *shutdownType)
 static void
 halt_button_clicked (GtkWidget *b, gint *shutdownType)
 {
-    *shutdownType = SHUTDOWN_HALT;
+    *shutdownType = XFSM_SHUTDOWN_HALT;
 
     gtk_dialog_response (GTK_DIALOG (shutdown_dialog), GTK_RESPONSE_OK);
 }
@@ -137,7 +141,7 @@ halt_button_clicked (GtkWidget *b, gint *shutdownType)
 /*
  */
 gboolean
-shutdownDialog(gint *shutdownType, gboolean *saveSession)
+shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolean *saveSession)
 {
   gboolean accessibility;
   XfsmFadeout *fadeout = NULL;
@@ -166,7 +170,7 @@ shutdownDialog(gint *shutdownType, gboolean *saveSession)
   gint result;
   XfceKiosk *kiosk;
   gboolean kiosk_can_shutdown;
-  XfceRc *rc;
+  XfconfChannel *channel;
 #ifdef SESSION_SCREENSHOTS
   GdkRectangle screenshot_area;
   GdkWindow *root;
@@ -190,18 +194,16 @@ shutdownDialog(gint *shutdownType, gboolean *saveSession)
   xfce_kiosk_free (kiosk);
 
   /* load configuration */
-  rc = xfsm_open_config (FALSE);
-  xfce_rc_set_group (rc, "General");
-  saveonexit = xfce_rc_read_bool_entry (rc, "SaveOnExit", TRUE);
-  autosave = xfce_rc_read_bool_entry (rc, "AutoSave", FALSE);
-  prompt = xfce_rc_read_bool_entry (rc, "PromptOnLogout", TRUE);
+  channel = xfsm_open_config ();
+  channel = xfconf_channel_get ("xfce4-session");
+  saveonexit = xfconf_channel_get_bool (channel, "/general/SaveOnExit", TRUE);
+  autosave = xfconf_channel_get_bool (channel, "/general/AutoSave", FALSE);
+  prompt = xfconf_channel_get_bool (channel, "/general/PromptOnLogout", TRUE);
 
   /* if PromptOnLogout is off, saving depends on AutoSave */
   if (!prompt)
     {
-      xfce_rc_close (rc);
-
-      *shutdownType = SHUTDOWN_LOGOUT;
+      *shutdownType = XFSM_SHUTDOWN_LOGOUT;
       *saveSession = autosave;
 
       return TRUE;
@@ -443,7 +445,7 @@ shutdownDialog(gint *shutdownType, gboolean *saveSession)
   gtk_widget_hide (dialog);
 
   /* ask password */
-  if (result == GTK_RESPONSE_OK && *shutdownType != SHUTDOWN_LOGOUT
+  if (result == GTK_RESPONSE_OK && *shutdownType != XFSM_SHUTDOWN_LOGOUT
       && xfsm_shutdown_helper_need_password (shutdown_helper))
     {
       gtk_widget_show (ok_button);
@@ -548,9 +550,8 @@ shutdownDialog(gint *shutdownType, gboolean *saveSession)
    */
   if (result == GTK_RESPONSE_OK)
     {
-      xfce_rc_set_group (rc, "General");
-      xfce_rc_write_entry (rc, "SessionName", session_name);
-      xfce_rc_write_bool_entry (rc, "SaveOnExit", *saveSession);
+      xfconf_channel_set_string (channel, "/general/SessionName", sessionName);
+      xfconf_channel_set_bool (channel, "/general/SaveOnExit", *saveSession);
     }
   else
     {
@@ -562,14 +563,12 @@ shutdownDialog(gint *shutdownType, gboolean *saveSession)
   if (screenshot_pm != NULL)
     {
       if (result == GTK_RESPONSE_OK)
-        screenshot_save (screenshot_pm, &screenshot_area);
+        screenshot_save (sessionName, screenshot_pm, &screenshot_area);
 
       g_object_unref (G_OBJECT (screenshot_pm));
     }
 #endif
 
-  xfce_rc_close (rc);
-
   return (result == GTK_RESPONSE_OK);
 }
 
@@ -577,25 +576,38 @@ shutdownDialog(gint *shutdownType, gboolean *saveSession)
 /*
  */
 gint
-xfsm_shutdown(gint type)
+xfsm_shutdown(XfsmShutdownType type)
 {
   gboolean result;
 
-  if (compat_gnome)
-    xfsm_compat_gnome_shutdown ();
+  /* kludge */
+  if (type == XFSM_SHUTDOWN_ASK)
+    {
+      g_warning ("xfsm_shutdown () passed XFSM_SHUTDOWN_ASK.  This is a bug.");
+      type = XFSM_SHUTDOWN_LOGOUT;
+    }
 
-  if (compat_kde)
-    xfsm_compat_kde_shutdown ();
+  /* these two remember if they were started or not */
+  xfsm_compat_gnome_shutdown ();
+  xfsm_compat_kde_shutdown ();
 
   /* kill legacy clients */
   xfsm_legacy_shutdown ();
 
-#ifdef HAVE_SYNC
-  /* sync disk block in-core status with that on disk */
-  sync ();
-#endif
+#if !defined(__NR_ioprio_set) && defined(HAVE_SYNC)
+  /* sync disk block in-core status with that on disk.  if
+   * we have ioprio_set (), then we've already synced. */
+  if (fork () == 0)
+    {
+# ifdef HAVE_SETSID
+      setsid ();
+# endif
+      sync ();
+      _exit (EXIT_SUCCESS);
+    }
+#endif  /* HAVE_SYNC */
 
-  if (type == SHUTDOWN_LOGOUT)
+  if (type == XFSM_SHUTDOWN_LOGOUT)
     return EXIT_SUCCESS;
 
   if (shutdown_helper == NULL)
@@ -604,15 +616,15 @@ xfsm_shutdown(gint type)
       return EXIT_FAILURE;
     }
 
-  if (type == SHUTDOWN_HALT)
+  if (type == XFSM_SHUTDOWN_HALT)
     {
       result = xfsm_shutdown_helper_send_command (shutdown_helper,
-                                                  XFSM_SHUTDOWN_POWEROFF);
+                                                  XFSM_SHUTDOWN_COMMAND_POWEROFF);
     }
   else
     {
       result = xfsm_shutdown_helper_send_command (shutdown_helper,
-                                                  XFSM_SHUTDOWN_REBOOT);
+                                                  XFSM_SHUTDOWN_COMMAND_REBOOT);
     }
 
   xfsm_shutdown_helper_destroy (shutdown_helper);
