@@ -1,4 +1,4 @@
-/* $Id: sm-layer.c 20833 2006-04-09 18:08:17Z benny $ */
+/* $Id: sm-layer.c 28142 2008-10-11 11:10:44Z kelnos $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -41,6 +41,7 @@
 #include <xfce4-session/xfsm-global.h>
 #include <xfce4-session/xfsm-manager.h>
 
+#define XFSM_CLIENT_MANAGER(c)  (XFSM_MANAGER (g_object_get_data (G_OBJECT (c), "--xfsm-manager")))
 
 /* local prototypes */
 static Status sm_new_client                 (SmsConn         sms_conn,
@@ -97,13 +98,14 @@ static IceListenObj *listen_objs;
 
 
 void
-sm_init (XfceRc  *rc,
-         gboolean disable_tcp)
+sm_init (XfconfChannel *channel,
+         gboolean       disable_tcp,
+         XfsmManager   *manager)
 {
   char *network_idlist;
   char  error[2048];
 
-  if (disable_tcp || xfce_rc_read_bool_entry (rc, "DisableTcp", FALSE))
+  if (disable_tcp || !xfconf_channel_get_bool (channel, "/security/EnableTcp", TRUE))
     {
 #ifdef HAVE__ICETRANSNOLISTEN
       extern void _IceTransNoListen (char *protocol);
@@ -119,7 +121,7 @@ sm_init (XfceRc  *rc,
 #endif
     }
   
-  if (!SmsInitialize (PACKAGE, VERSION, sm_new_client, NULL, ice_auth_proc,
+  if (!SmsInitialize (PACKAGE, VERSION, sm_new_client, manager, ice_auth_proc,
                       2048, error))
     {
       fprintf (stderr, "xfce4-session: Unable to register XSM protocol: %s\n", error);
@@ -132,7 +134,7 @@ sm_init (XfceRc  *rc,
       exit (EXIT_FAILURE);
     }
   
-  ice_setup_listeners (num_listeners, listen_objs);
+  ice_setup_listeners (num_listeners, listen_objs, manager);
   
   network_idlist = IceComposeNetworkIdList (num_listeners, listen_objs);
   xfce_setenv ("SESSION_MANAGER", network_idlist, TRUE);
@@ -147,13 +149,14 @@ sm_new_client (SmsConn        sms_conn,
                SmsCallbacks  *callbacks,
                char         **failure_reason)
 {
-  XfsmClient *client;
-  gchar      *error = NULL;
+  XfsmManager *manager = XFSM_MANAGER (manager_data);
+  XfsmClient  *client;
+  gchar       *error = NULL;
 
   xfsm_verbose ("ICE connection fd = %d, received NEW CLIENT\n\n",
                 IceConnectionNumber (SmsGetIceConnection (sms_conn)));
   
-  client = xfsm_manager_new_client (sms_conn, &error);
+  client = xfsm_manager_new_client (manager, sms_conn, &error);
   if (client == NULL)
     {
       xfsm_verbose ("NEW CLIENT failed: %s\n", error);
@@ -185,6 +188,8 @@ sm_new_client (SmsConn        sms_conn,
     | SmsSaveYourselfP2RequestProcMask | SmsSaveYourselfDoneProcMask
     | SmsCloseConnectionProcMask | SmsSetPropertiesProcMask
     | SmsDeletePropertiesProcMask | SmsGetPropertiesProcMask;
+
+  g_object_set_data (G_OBJECT (client), "--xfsm-manager", manager);
                                                                                 
   return True;
 }
@@ -202,7 +207,7 @@ sm_register_client (SmsConn   sms_conn,
                 IceConnectionNumber (SmsGetIceConnection (sms_conn)),
                 previous_id != NULL ? previous_id : "None");
   
-  result = xfsm_manager_register_client (client, previous_id);
+  result = xfsm_manager_register_client (XFSM_CLIENT_MANAGER (client), client, previous_id);
 
   if (previous_id != NULL)
     free (previous_id);
@@ -219,9 +224,9 @@ sm_interact_request (SmsConn   sms_conn,
   XfsmClient *client = (XfsmClient *) client_data;
 
   xfsm_verbose ("Client Id = %s, received INTERACT REQUEST [Dialog type = %s]\n\n",
-                client->id, dialog_type == SmDialogError ? "Error" : "Normal");
+                xfsm_client_get_id (client), dialog_type == SmDialogError ? "Error" : "Normal");
   
-  xfsm_manager_interact (client, dialog_type);
+  xfsm_manager_interact (XFSM_CLIENT_MANAGER (client), client, dialog_type);
 }
 
 
@@ -233,9 +238,9 @@ sm_interact_done (SmsConn   sms_conn,
   XfsmClient *client = (XfsmClient *) client_data;
 
   xfsm_verbose ("Client Id = %s, received INTERACT DONE [Cancel shutdown = %s]\n\n",
-                client->id, cancel_shutdown ? "True" : "False");
+                xfsm_client_get_id (client), cancel_shutdown ? "True" : "False");
   
-  xfsm_manager_interact_done (client, cancel_shutdown);
+  xfsm_manager_interact_done (XFSM_CLIENT_MANAGER (client), client, cancel_shutdown);
 }
 
 
@@ -253,7 +258,7 @@ sm_save_yourself_request (SmsConn   sms_conn,
   if (G_UNLIKELY (verbose))
     {
       xfsm_verbose ("Client Id = %s, received SAVE YOURSELF REQUEST\n",
-                    client->id);
+                    xfsm_client_get_id (client));
       xfsm_verbose ("   Save type:      %s\n",
                     save_type == SmSaveLocal ? "Local"
                     : (save_type == SmSaveGlobal ? "Global" : "Both"));
@@ -266,7 +271,7 @@ sm_save_yourself_request (SmsConn   sms_conn,
       xfsm_verbose ("\n");
     }
   
-  xfsm_manager_save_yourself (client, save_type, shutdown, interact_style, fast, global);
+  xfsm_manager_save_yourself (XFSM_CLIENT_MANAGER (client), client, save_type, shutdown, interact_style, fast, global);
 }
 
 
@@ -276,9 +281,10 @@ sm_save_yourself_phase2_request (SmsConn   sms_conn,
 {
   XfsmClient *client = (XfsmClient *) client_data;
 
-  xfsm_verbose ("Client Id = %s, received SAVE YOURSELF PHASE2 REQUEST\n\n", client->id);
+  xfsm_verbose ("Client Id = %s, received SAVE YOURSELF PHASE2 REQUEST\n\n",
+                xfsm_client_get_id (client));
   
-  xfsm_manager_save_yourself_phase2 (client);
+  xfsm_manager_save_yourself_phase2 (XFSM_CLIENT_MANAGER (client), client);
 }
 
 
@@ -290,9 +296,9 @@ sm_save_yourself_done (SmsConn   sms_conn,
   XfsmClient *client = (XfsmClient *) client_data;
 
   xfsm_verbose ("Client Id = %s, received SAVE YOURSELF DONE [Success = %s]\n\n",
-                client->id, success ? "True" : "False");
+                xfsm_client_get_id (client), success ? "True" : "False");
   
-  xfsm_manager_save_yourself_done (client, success);
+  xfsm_manager_save_yourself_done (XFSM_CLIENT_MANAGER (client), client, success);
 }
 
 
@@ -308,13 +314,13 @@ sm_close_connection (SmsConn   sms_conn,
   if (G_UNLIKELY (verbose))
     {
       xfsm_verbose ("Client Id = %s, received CLOSE CONNECTION [Num reasons = %d]\n",
-                    client->id, num_reasons);
+                    xfsm_client_get_id (client), num_reasons);
       for (n = 0; n < num_reasons; ++n)
         xfsm_verbose ("   Reason %2d: %s\n", n, reasons[n]);
       xfsm_verbose ("\n");
     }
   
-  xfsm_manager_close_connection (client, TRUE);
+  xfsm_manager_close_connection (XFSM_CLIENT_MANAGER (client), client, TRUE);
 
   if (num_reasons > 0)
     SmFreeReasons (num_reasons, reasons);
@@ -327,15 +333,14 @@ sm_set_properties (SmsConn   sms_conn,
                    int       num_props,
                    SmProp  **props)
 {
-  XfsmClient     *client     = (XfsmClient *) client_data;
-  XfsmProperties *properties = client->properties;
-  int             n;
-  int             i;
+  XfsmClient *client = (XfsmClient *) client_data;
+  int         n;
+  int         i;
   
   if (G_UNLIKELY (verbose))
     {
       xfsm_verbose ("Client Id = %s, received SET PROPERTIES [Num props = %d]\n",
-                    properties->client_id, num_props);
+                    xfsm_client_get_id (client), num_props);
       for (n = 0; n < num_props; ++n)
         {
           xfsm_verbose ("   Name:  %s\n   Type:  %s\n", props[n]->name, props[n]->type);
@@ -361,7 +366,7 @@ sm_set_properties (SmsConn   sms_conn,
       xfsm_verbose ("\n");
     }  
   
-  xfsm_properties_merge (properties, num_props, props);
+  xfsm_client_merge_properties (client, props, num_props);
   
   while (num_props-- > 0)
     SmFreeProperty (props[num_props]);
@@ -375,20 +380,19 @@ sm_delete_properties (SmsConn   sms_conn,
                       int       num_props,
                       char    **prop_names)
 {
-  XfsmClient     *client     = (XfsmClient *) client_data;
-  XfsmProperties *properties = client->properties;
-  int             n;
+  XfsmClient *client = (XfsmClient *) client_data;
+  int         n;
 
   if (G_UNLIKELY (verbose))
     {
       xfsm_verbose ("Client Id = %s, received DELETE PROPERTIES [Num props = %d]\n",
-                    properties->client_id, num_props);
+                    xfsm_client_get_id (client), num_props);
       for (n = 0; n < num_props; ++n)
         xfsm_verbose ("   Name:   %s\n", prop_names[n]);
       xfsm_verbose ("\n");
     }
   
-  xfsm_properties_delete (properties, num_props, prop_names);
+  xfsm_client_delete_properties (client, prop_names, num_props);
   
   while (num_props-- > 0)
     free (prop_names[num_props]);
@@ -401,13 +405,13 @@ sm_get_properties (SmsConn   sms_conn,
                    SmPointer client_data)
 {
   XfsmClient     *client     = (XfsmClient *) client_data;
-  XfsmProperties *properties = client->properties;
+  XfsmProperties *properties = xfsm_client_get_properties (client);
   SmProp        **props      = NULL;
   gint            num_props  = 0;
 
   xfsm_verbose ("Client Id = %s, received GET PROPERTIES\n\n", properties->client_id);
   
-  xfsm_properties_extract (client->properties, &num_props, &props);
+  xfsm_properties_extract (properties, &num_props, &props);
 
   SmsReturnProperties (sms_conn, num_props, props);
   
