@@ -1,6 +1,7 @@
-/* $Id: xfsm-properties.c,v 1.6 2004/11/01 16:10:55 benny Exp $ */
+/* $Id: xfsm-properties.c 28064 2008-10-06 23:10:22Z kelnos $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -46,13 +47,10 @@ static SmProp* str_to_property  (const gchar  *name,
                                  const gchar  *value) G_GNUC_PURE;
 static SmProp* int_to_property  (const gchar  *name,
                                  gint          value) G_GNUC_PURE;
-static gchar** property_to_strv (const SmProp *prop)  G_GNUC_PURE;
-static void    xfsm_properties_replace_discard_command (XfsmProperties *properties,
-                                                        gchar         **new_discard);
 
 
 #ifndef HAVE_STRDUP
-G_INLINE_FUNC char*
+static char*
 strdup (const char *s)
 {
   char *t;
@@ -66,7 +64,7 @@ strdup (const char *s)
 #endif
 
 
-G_INLINE_FUNC gchar*
+static gchar*
 compose (gchar       *buffer,
          gsize        length,
          const gchar *prefix,
@@ -145,60 +143,6 @@ int_to_property (const gchar *name,
 }
 
 
-static gchar**
-property_to_strv (const SmProp *prop)
-{
-  gchar **strv = NULL;
-  gint    strc;
-  guint   n;
-  
-  if (strcmp (prop->type, SmARRAY8) == 0)
-    {
-      if (!g_shell_parse_argv ((const gchar *) prop->vals->value,
-                               &strc, &strv, NULL))
-        return NULL;
-    }
-  else if (strcmp (prop->type, SmLISTofARRAY8) == 0)
-    {
-      strv = g_new (gchar *, prop->num_vals + 1);
-      for (n = 0; n < prop->num_vals; ++n)
-        strv[n] = g_strdup ((const gchar *) prop->vals[n].value);
-      strv[n] = NULL;
-    }
-
-  return strv;
-}
-
-
-static void
-xfsm_properties_replace_discard_command (XfsmProperties *properties,
-                                         gchar         **new_discard)
-{
-  gchar **old_discard = properties->discard_command;
-
-  if (old_discard != NULL)
-    {
-      if (!xfsm_strv_equal (old_discard, new_discard))
-        {
-          xfsm_verbose ("Client Id = %s, running old discard command.\n\n",
-                        properties->client_id);
-
-          g_spawn_sync (properties->current_directory,
-                        old_discard,
-                        properties->environment,
-                        G_SPAWN_SEARCH_PATH,
-                        NULL, NULL,
-                        NULL, NULL,
-                        NULL, NULL);
-        }
-
-      g_strfreev (old_discard);
-    }
-
-  properties->discard_command = new_discard;
-}
-
-
 XfsmProperties*
 xfsm_properties_new (const gchar *client_id,
                      const gchar *hostname)
@@ -214,67 +158,6 @@ xfsm_properties_new (const gchar *client_id,
 }
 
 
-void
-xfsm_properties_delete (XfsmProperties *properties,
-                        gint            num_props,
-                        gchar         **prop_names)
-{
-  gint n;
-  
-  for (n = 0; n < num_props; ++n)
-    {
-      if (strcmp (prop_names[n], SmCloneCommand) == 0)
-        {
-          if (properties->clone_command != NULL)
-            {
-              g_strfreev (properties->clone_command);
-              properties->clone_command = NULL;
-            }
-        }
-      else if (strcmp (prop_names[n], SmCurrentDirectory) == 0)
-        {
-          if (properties->current_directory != NULL)
-            {
-              g_free (properties->current_directory);
-              properties->current_directory = NULL;
-            }
-        }
-      else if (strcmp (prop_names[n], SmDiscardCommand) == 0)
-        {
-          if (properties->discard_command != NULL)
-            {
-              g_strfreev (properties->discard_command);
-              properties->discard_command = NULL;
-            }
-        }
-      else if (strcmp (prop_names[n], SmEnvironment) == 0)
-        {
-          if (properties->environment != NULL)
-            {
-              g_strfreev (properties->environment);
-              properties->environment = NULL;
-            }
-        }
-      else if (strcmp (prop_names[n], GsmPriority) == 0)
-        {
-          properties->priority = 50;
-        }
-      else if (strcmp (prop_names[n], SmRestartStyleHint) == 0)
-        {
-          properties->restart_style_hint = SmRestartIfRunning;
-        }
-      else if (strcmp (prop_names[n], SmUserID) == 0)
-        {
-          if (properties->user_id != NULL)
-            {
-              g_free (properties->user_id);
-              properties->user_id = NULL;
-            }
-        }
-    }
-}
-
-
 void
 xfsm_properties_extract (XfsmProperties *properties,
                          gint           *num_props,
@@ -301,14 +184,23 @@ xfsm_properties_extract (XfsmProperties *properties,
 
   *pp++ = int_to_property (GsmPriority, properties->priority);
 
+  if (properties->process_id != NULL)
+    *pp++ = str_to_property (SmProcessID, properties->process_id);
+
   if (properties->program != NULL)
     *pp++ = str_to_property (SmProgram, properties->program);
   
+  if (properties->resign_command != NULL)
+    *pp++ = strv_to_property (SmResignCommand, properties->resign_command);
+
   if (properties->restart_command != NULL)
     *pp++ = strv_to_property (SmRestartCommand, properties->restart_command);
   
   *pp++ = int_to_property (SmRestartStyleHint, properties->restart_style_hint);
   
+  if (properties->shutdown_command != NULL)
+    *pp++ = strv_to_property (SmShutdownCommand, properties->shutdown_command);
+
   if (properties->user_id != NULL)
     *pp++ = str_to_property (SmUserID, properties->user_id);
   
@@ -316,185 +208,6 @@ xfsm_properties_extract (XfsmProperties *properties,
 }
 
 
-void
-xfsm_properties_merge (XfsmProperties *properties,
-                       gint            num_props,
-                       SmProp        **props)
-{
-  gchar **strv;
-  SmProp *prop;
-  guint   n;
-  
-  for (n = 0; n < num_props; ++n)
-    {
-      prop = props[n];
-
-      if (strcmp (prop->name, SmCloneCommand) == 0)
-        {
-          strv = property_to_strv (prop);
-          
-          if (strv != NULL)
-            {
-              if (properties->clone_command != NULL)
-                g_strfreev (properties->clone_command);
-              properties->clone_command = strv;
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, SmCurrentDirectory) == 0)
-        {
-          if (properties->current_directory != NULL)
-            g_free (properties->current_directory);
-          properties->current_directory = g_strdup ((const gchar *) prop->vals->value);
-        }
-      else if (strcmp (prop->name, SmDiscardCommand) == 0)
-        {
-          strv = property_to_strv (prop);
-          
-          if (strv != NULL)
-            {
-              xfsm_properties_replace_discard_command (properties, strv);
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, SmEnvironment) == 0)
-        {
-          strv = property_to_strv (prop);
-
-          if (strv != NULL)
-            {
-              if (properties->environment != NULL)
-                g_strfreev (properties->environment);
-              properties->environment = strv;
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, GsmPriority) == 0)
-        {
-          if (strcmp (prop->type, SmCARD8) == 0)
-            {
-              properties->priority = *((gint8 *) prop->vals->value);
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, SmProgram) == 0)
-        {
-          if (strcmp (prop->type, SmARRAY8) == 0)
-            {
-              if (properties->program != NULL)
-                g_free (properties->program);
-
-              /* work-around damn f*cking xmms */
-              if (properties->restart_command != NULL
-                  && g_str_has_suffix (properties->restart_command[0], "xmms"))
-                {
-                  properties->program = g_strdup ("xmms");
-                }
-              else
-                {
-                  properties->program = g_strdup ((const gchar *) prop->vals->value);
-                }
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, SmRestartCommand) == 0)
-        {
-          strv = property_to_strv (prop);
-          
-          if (strv != NULL)
-            {
-              if (properties->restart_command != NULL)
-                g_strfreev (properties->restart_command);
-              properties->restart_command = strv;
-
-              /* work-around damn f*cking xmms */
-              if (g_str_has_suffix (strv[0], "xmms"))
-                {
-                  if (properties->program != NULL)
-                    g_free (properties->program);
-                  properties->program = g_strdup ("xmms");
-                }
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, SmRestartStyleHint) == 0)
-        {
-          if (strcmp (prop->type, SmCARD8) == 0)
-            {
-              properties->restart_style_hint = *((gint8 *) prop->vals->value);
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-      else if (strcmp (prop->name, SmUserID) == 0)
-        {
-          if (strcmp (prop->type, SmARRAY8) == 0)
-            {
-              if (properties->user_id != NULL)
-                g_free (properties->user_id);
-              properties->user_id = g_strdup ((const gchar *) prop->vals->value);
-            }
-          else
-            {
-              g_warning ("Client %s specified property %s of invalid "
-                         "type %s, ignoring.",
-                         properties->client_id,
-                         prop->name,
-                         prop->type);
-            }
-        }
-    }
-}
-
-
 XfsmProperties*
 xfsm_properties_load (XfceRc      *rc,
                       const gchar *prefix)
@@ -533,8 +246,12 @@ xfsm_properties_load (XfceRc      *rc,
                                                             NULL);
   properties->environment        = xfce_rc_read_list_entry (rc, ENTRY ("Environment"),
                                                             NULL);
+  properties->resign_command     = xfce_rc_read_list_entry (rc, ENTRY ("ResignCOmmand"),
+                                                            NULL);
   properties->restart_command    = xfce_rc_read_list_entry (rc, ENTRY ("RestartCommand"),
                                                             NULL);
+  properties->shutdown_command   = xfce_rc_read_list_entry (rc, ENTRY ("ShutdownCommand"),
+                                                            NULL);
   properties->priority           = xfce_rc_read_int_entry (rc, ENTRY ("Priority"), 50);
   properties->restart_style_hint = xfce_rc_read_int_entry (rc, ENTRY ("RestartStyleHint"),
                                                            SmRestartIfRunning);
@@ -605,12 +322,20 @@ xfsm_properties_store (XfsmProperties *properties,
                                properties->priority);
     }
 
+  /* ProcessID isn't something you'd generally want saved... */
+
   if (properties->program != NULL)
     {
       xfce_rc_write_entry (rc, ENTRY ("Program"),
                            properties->program);
     }
-  
+
+  if (properties->resign_command != NULL)
+    {
+      xfce_rc_write_list_entry (rc, ENTRY ("ResignCommand"),
+                                properties->resign_command, NULL);
+    }
+
   if (properties->restart_command != NULL)
     {
       xfce_rc_write_list_entry (rc, ENTRY ("RestartCommand"),
@@ -622,6 +347,12 @@ xfsm_properties_store (XfsmProperties *properties,
       xfce_rc_write_int_entry (rc, ENTRY ("RestartStyleHint"),
                                properties->restart_style_hint);
     }
+
+  if (properties->shutdown_command != NULL)
+    {
+      xfce_rc_write_list_entry (rc, ENTRY ("ShutdownCommand"),
+                                properties->shutdown_command, NULL);
+    }
   
   if (properties->user_id != NULL)
     {
@@ -658,6 +389,8 @@ xfsm_properties_free (XfsmProperties *properties)
 {
   g_return_if_fail (properties != NULL);
 
+  if (properties->startup_timeout_id != 0)
+    g_source_remove (properties->startup_timeout_id);
   if (properties->client_id != NULL)
     g_free (properties->client_id);
   if (properties->hostname != NULL)
@@ -666,15 +399,117 @@ xfsm_properties_free (XfsmProperties *properties)
     g_strfreev (properties->clone_command);
   if (properties->current_directory != NULL)
     g_free (properties->current_directory);
+  if (properties->process_id != NULL)
+    g_free (properties->process_id);
   if (properties->program != NULL)
     g_free (properties->program);
   if (properties->discard_command != NULL)
     g_strfreev (properties->discard_command);
+  if (properties->resign_command != NULL)
+    g_strfreev (properties->resign_command);
   if (properties->restart_command != NULL)
     g_strfreev (properties->restart_command);
+  if (properties->shutdown_command != NULL)
+    g_strfreev (properties->shutdown_command);
   if (properties->environment != NULL)
     g_strfreev (properties->environment);
   if (properties->user_id)
     g_free (properties->user_id);
   g_free (properties);
 }
+
+
+gchar **
+xfsm_strv_from_smprop (const SmProp *prop)
+{
+  gchar **strv = NULL;
+  gint    strc;
+  guint   n;
+  
+  if (strcmp (prop->type, SmARRAY8) == 0)
+    {
+      if (!g_shell_parse_argv ((const gchar *) prop->vals->value,
+                               &strc, &strv, NULL))
+        return NULL;
+    }
+  else if (strcmp (prop->type, SmLISTofARRAY8) == 0)
+    {
+      strv = g_new (gchar *, prop->num_vals + 1);
+      for (n = 0; n < prop->num_vals; ++n)
+        strv[n] = g_strdup ((const gchar *) prop->vals[n].value);
+      strv[n] = NULL;
+    }
+
+  return strv;
+}
+
+
+GValue *
+xfsm_g_value_from_property (XfsmProperties *properties,
+                            const gchar *name)
+{
+  GValue *val = NULL;
+
+  if (strcmp (name, SmCloneCommand) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRV);
+      g_value_take_boxed (val, g_strdupv (properties->clone_command));
+    }
+  else if (strcmp (name, SmCurrentDirectory) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRING);
+      g_value_take_string (val, g_strdup (properties->current_directory));
+    }
+  else if (strcmp (name, SmDiscardCommand) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRV);
+      g_value_take_boxed (val, g_strdupv (properties->discard_command));
+    }
+  else if (strcmp (name, SmEnvironment) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRV);
+      g_value_take_boxed (val, g_strdupv (properties->environment));
+    }
+  else if (strcmp(name, SmProcessID) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRING);
+      g_value_take_string (val, g_strdup (properties->process_id));
+    }
+  else if (strcmp (name, SmProgram) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRING);
+      g_value_take_string (val, g_strdup (properties->program));
+    }
+  else if (strcmp (name, SmRestartCommand) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRV);
+      g_value_take_boxed (val, g_strdupv (properties->restart_command));
+    }
+  else if (strcmp (name, SmResignCommand) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRV);
+      g_value_take_boxed (val, g_strdupv (properties->resign_command));
+    }
+  else if (strcmp (name, SmRestartStyleHint) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_UCHAR);
+      g_value_set_uchar (val, properties->restart_style_hint);
+    }
+  else if (strcmp (name, SmShutdownCommand) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRV);
+      g_value_take_boxed (val, g_strdupv (properties->shutdown_command));
+    }
+  else if (strcmp (name, SmUserID) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_STRING);
+      g_value_take_string (val, g_strdup (properties->user_id));
+    }
+  else if (strcmp (name, GsmPriority) == 0)
+    {
+      val = xfsm_g_value_new (G_TYPE_UCHAR);
+      g_value_set_uchar (val, properties->priority);
+    }
+
+  return val;
+}
