@@ -1,6 +1,7 @@
-/* $Id: xfsm-startup.c 23146 2006-09-13 20:06:43Z benny $ */
+/* $Id: xfsm-startup.c 28142 2008-10-11 11:10:44Z kelnos $ */
 /*-
  * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -55,21 +56,42 @@
 #include <xfce4-session/xfsm-manager.h>
 #include <xfce4-session/xfsm-splash-screen.h>
 
+#include "xfsm-startup.h"
+
+
+typedef struct
+{
+  XfsmManager    *manager;
+  XfsmProperties *properties;
+} XfsmStartupTimeoutData;
+
+typedef struct
+{
+  XfsmManager *manager;
+  gchar *client_id;
+} XfsmStartupChildWatchData;
 
 
 /*
    Prototypes
  */
-static gboolean xfsm_startup_continue_failsafe    (void);
-static gboolean xfsm_startup_continue_session     (const gchar *previous_id);
-static void     xfsm_startup_child_watch          (GPid         pid,
-                                                   gint         status,
-                                                   gpointer     user_data);
-static void     xfsm_startup_child_watch_destroy  (gpointer     user_data);
+static void     xfsm_startup_failsafe                (XfsmManager *manager);
 
+static gboolean xfsm_startup_session_next_prio_group (XfsmManager *manager);
+
+static void     xfsm_startup_child_watch             (GPid         pid,
+                                                      gint         status,
+                                                      gpointer     user_data);
+static void     xfsm_startup_child_watch_destroy     (gpointer     user_data);
+
+static gboolean xfsm_startup_timeout                 (gpointer     data);
+static void     xfsm_startup_timeout_destroy         (gpointer     data);
+
+static void xfsm_startup_handle_failed_client        (XfsmProperties *properties,
+                                                      XfsmManager    *manager);
 
 void
-xfsm_startup_init (XfceRc *rc)
+xfsm_startup_init (XfconfChannel *channel)
 {
   /* nothing to be done here, currently */
 }
@@ -419,43 +441,40 @@ xfsm_startup_autostart (void)
 
 
 void
-xfsm_startup_foreign (void)
+xfsm_startup_foreign (XfsmManager *manager)
 {
-  if (compat_kde)
+  if (xfsm_manager_get_compat_startup(manager, XFSM_MANAGER_COMPAT_KDE))
     xfsm_compat_kde_startup (splash_screen);
 
-  if (compat_gnome)
+  if (xfsm_manager_get_compat_startup(manager, XFSM_MANAGER_COMPAT_GNOME))
     xfsm_compat_gnome_startup (splash_screen);
 }
 
 
-/* Returns TRUE if done, else FALSE */
-gboolean
-xfsm_startup_continue (const gchar *previous_id)
+void
+xfsm_startup_begin (XfsmManager *manager)
 {
-  gboolean startup_done = FALSE;
-
-  if (failsafe_mode)
-    startup_done = xfsm_startup_continue_failsafe ();
+  if (xfsm_manager_get_use_failsafe_mode (manager))
+    {
+      xfsm_startup_failsafe (manager);
+      xfsm_startup_autostart ();
+    }
   else
-    startup_done = xfsm_startup_continue_session (previous_id);
-
-  /* perform Autostart */
-  if (startup_done)
-    xfsm_startup_autostart ();
-  
-  return startup_done;
+    {
+      xfsm_startup_session_continue (manager);
+    }
 }
 
 
-static gboolean
-xfsm_startup_continue_failsafe (void)
-{  
+static void
+xfsm_startup_failsafe (XfsmManager *manager)
+{
+  GQueue *failsafe_clients = xfsm_manager_get_queue (manager, XFSM_MANAGER_QUEUE_FAILSAFE_CLIENTS);
   FailsafeClient *fclient;
-  
-  fclient = (FailsafeClient *) g_list_nth_data (failsafe_clients, 0);
-  if (fclient != NULL)
+
+  while ((fclient = g_queue_pop_head (failsafe_clients)))
     {
+      /* FIXME: splash */
       /* let the user know whats going on */
       if (G_LIKELY (splash_screen != NULL))
         {
@@ -466,89 +485,148 @@ xfsm_startup_continue_failsafe (void)
       /* start the application */
       xfsm_start_application (fclient->command, NULL, fclient->screen,
                               NULL, NULL, NULL);
-      failsafe_clients = g_list_remove (failsafe_clients, fclient);
-      g_strfreev (fclient->command);
-      g_free (fclient);
-
-      /* there are more to come */
-      return FALSE;
+      xfsm_failsafe_client_free (fclient);
     }
-
-  return TRUE;
 }
 
 
-static gboolean
-xfsm_startup_continue_session (const gchar *previous_id)
+static GPid
+xfsm_startup_session_client (XfsmProperties *properties)
 {
-  XfsmProperties *properties;
-  gchar         **argv;
-  gint            argc;
-  GPid            pid;
-  gint            n;
-  
-again:
-  properties = (XfsmProperties *) g_list_nth_data (pending_properties, 0);
-  if (properties != NULL)
+  gchar **argv;
+  gint    argc;
+  GPid    pid;
+  gint    n;
+
+  /* generate the argument vector for the application (expanding variables) */
+  argc = g_strv_length (properties->restart_command);
+  argv = g_new (gchar *, argc + 1);
+  for (n = 0; n < argc; ++n)
+    argv[n] = xfce_expand_variables (properties->restart_command[n], NULL);
+  argv[n] = NULL;
+
+  /* fork a new process for the application */
+#ifdef HAVE_VFORK
+  pid = vfork ();
+#else
+  pid = fork ();
+#endif
+
+  /* handle the child process */
+  if (pid == 0)
     {
+      /* execute the application here */
+      execvp (argv[0], argv);
+      _exit (127);
+    }
+
+  /* cleanup */
+  g_strfreev (argv);
+
+  /* check if we failed to fork */
+  if (G_UNLIKELY (pid < 0))
+    {
+      /* tell the user that we failed to fork */
+      perror ("Failed to fork new process");
+      return -1;
+    }
+
+  return pid;
+}
+
+
+void
+xfsm_startup_session_continue (XfsmManager *manager)
+{
+  GQueue  *pending_properties = xfsm_manager_get_queue (manager, XFSM_MANAGER_QUEUE_PENDING_PROPS);
+  gboolean client_started = FALSE;
+
+  /* try to start some clients.  if we fail to start anything in the current
+   * priority group, move right to the next one.  if we *did* start something,
+   * the failed/registered handlers will take care of moving us on to the
+   * next priority group */
+  while (client_started == FALSE && g_queue_peek_head (pending_properties) != NULL)
+    client_started = xfsm_startup_session_next_prio_group (manager);
+
+  if (G_UNLIKELY (client_started == FALSE && g_queue_peek_head (pending_properties) == NULL))
+    {
+      /* we failed to start anything, and we don't have anything else,
+       * to start, so just move on to the autostart items and signal
+       * the manager that we're finished */
+      xfsm_startup_autostart ();
+      xfsm_manager_signal_startup_done (manager);
+    }
+}
+
+
+/* returns TRUE if we started anything, FALSE if we didn't */
+static gboolean
+xfsm_startup_session_next_prio_group (XfsmManager *manager)
+{
+  GQueue         *pending_properties = xfsm_manager_get_queue (manager, XFSM_MANAGER_QUEUE_PENDING_PROPS);
+  GQueue         *starting_properties = xfsm_manager_get_queue (manager, XFSM_MANAGER_QUEUE_STARTING_PROPS);
+  XfsmProperties *properties;
+  gint            cur_prio_group;
+  gboolean        client_started = FALSE;
+  GPid            pid;
+
+  properties = (XfsmProperties *) g_queue_peek_head (pending_properties);
+  if (properties == NULL)
+    return FALSE;
+
+  cur_prio_group = properties->priority;
+
+  xfsm_verbose ("Starting apps in prio group %d\n", cur_prio_group);
+
+  while ((properties = g_queue_pop_head (pending_properties)))
+    {
+      /* quit if we've hit all the clients in the current prio group */
+      if (properties->priority != cur_prio_group)
+        {
+          g_queue_push_head (pending_properties, properties);
+          break;
+        }
+
+      /* FIXME: splash */
       if (G_LIKELY (splash_screen != NULL))
         {
           xfsm_splash_screen_next (splash_screen,
                                    figure_app_name (properties->program));
         }
 
-      /* drop the properties from the pending list */
-      pending_properties = g_list_remove (pending_properties, properties);
-
-      /* generate the argument vector for the application (expanding variables) */
-      argc = g_strv_length (properties->restart_command);
-      argv = g_new (gchar *, argc + 1);
-      for (n = 0; n < argc; ++n)
-        argv[n] = xfce_expand_variables (properties->restart_command[n], NULL);
-      argv[n] = NULL;
-
-      /* fork a new process for the application */
-#ifdef HAVE_VFORK
-      pid = vfork ();
-#else
-      pid = fork ();
-#endif
-
-      /* handle the child process */
-      if (pid == 0)
+      if (G_LIKELY ((pid = xfsm_startup_session_client (properties)) != -1))
         {
-          /* execute the application here */
-          execvp (argv[0], argv);
-          _exit (127);
-        }
+          XfsmStartupChildWatchData *cwdata;
+          XfsmStartupTimeoutData    *stdata;
 
-      /* check if we failed to fork */
-      if (G_UNLIKELY (pid < 0))
-        {
-          /* tell the user that we failed to fork */
-          perror ("Failed to fork new process");
+          cwdata = g_new(XfsmStartupChildWatchData, 1);
+          cwdata->manager = g_object_ref (manager);
+          cwdata->client_id = g_strdup (properties->client_id);
+          g_child_watch_add_full (G_PRIORITY_LOW, pid,
+                                  xfsm_startup_child_watch, cwdata,
+                                  xfsm_startup_child_watch_destroy);
+
+          stdata = g_new(XfsmStartupTimeoutData, 1);
+          stdata->manager = manager;
+          stdata->properties = properties;
+          properties->startup_timeout_id = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                                               STARTUP_TIMEOUT,
+                                                               xfsm_startup_timeout,
+                                                               stdata,
+                                                               xfsm_startup_timeout_destroy);
+
+          g_queue_push_tail (starting_properties, properties);
+          client_started = TRUE;
         }
       else
         {
-          /* watch the child process */
-          g_child_watch_add_full (G_PRIORITY_LOW, pid, xfsm_startup_child_watch, g_strdup (properties->client_id), xfsm_startup_child_watch_destroy);
+          /* if starting the app failed, just ditch it */
+          xfsm_manager_handle_failed_client (manager, properties);
+          xfsm_properties_free (properties);
         }
-
-      /* cleanup */
-      g_strfreev (argv);
-
-      /* move the properties to the list of starting applications */
-      starting_properties = g_list_append (starting_properties, properties);
-
-      /* try with the next pending if fork() failed */
-      if (G_UNLIKELY (pid < 0))
-        goto again;
-
-      /* more to come... */
-      return FALSE;
     }
 
-  return TRUE;
+  return client_started;
 }
 
 
@@ -557,19 +635,23 @@ xfsm_startup_child_watch (GPid     pid,
                           gint     status,
                           gpointer user_data)
 {
-  XfsmProperties *properties;
-  const gchar    *client_id = user_data;
-  GList          *lp;
+  XfsmProperties            *properties;
+  XfsmStartupChildWatchData *cwdata = user_data;
+  GQueue                    *starting_properties;
+  gint                      i, n_items;
+
+  starting_properties = xfsm_manager_get_queue (cwdata->manager, XFSM_MANAGER_QUEUE_STARTING_PROPS);
+  n_items = g_queue_get_length (starting_properties);
 
   /* check if we have a starting process with the given client_id */
-  for (lp = starting_properties; lp != NULL; lp = lp->next)
+  for (i = 0; i < n_items; ++i)
     {
       /* check if this properties matches */
-      properties = (XfsmProperties *) lp->data;
-      if (strcmp (properties->client_id, client_id) == 0)
+      properties = (XfsmProperties *) g_queue_peek_nth (starting_properties, i);
+      if (strcmp (properties->client_id, cwdata->client_id) == 0)
         {
           /* continue startup, this client failed most probably */
-          xfsm_manager_startup_continue (NULL);
+          xfsm_startup_handle_failed_client (properties, cwdata->manager);
           break;
         }
     }
@@ -579,10 +661,49 @@ xfsm_startup_child_watch (GPid     pid,
 static void
 xfsm_startup_child_watch_destroy (gpointer user_data)
 {
-  /* release the client_id */
-  g_free (user_data);
+  XfsmStartupChildWatchData *cwdata = user_data;
+
+  g_object_unref (cwdata->manager);
+  g_free (cwdata->client_id);
+  g_free (cwdata);
 }
 
 
+static gboolean
+xfsm_startup_timeout (gpointer data)
+{
+  XfsmStartupTimeoutData *stdata = data;
+
+  stdata->properties->startup_timeout_id = 0;
+  xfsm_startup_handle_failed_client(stdata->properties, stdata->manager);
+
+  return FALSE;
+}
+
+
+static void
+xfsm_startup_timeout_destroy (gpointer data)
+{
+  g_free (data);
+}
+
+
+static void
+xfsm_startup_handle_failed_client (XfsmProperties *properties,
+                                   XfsmManager    *manager)
+{
+  GQueue *starting_properties = xfsm_manager_get_queue (manager, XFSM_MANAGER_QUEUE_STARTING_PROPS);
+
+  xfsm_manager_handle_failed_client (manager, properties);
+
+  g_queue_remove (starting_properties, properties);
+  xfsm_properties_free (properties);
+
+  if (g_queue_peek_head (starting_properties) == NULL)
+    {
+      /* everything has finished starting or failed; continue startup */
+      xfsm_startup_session_continue (manager);
+    }
+}
 
 
